/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.DisplayBean.NodeShape;

import java.util.Calendar;
import java.util.Vector;


/**
 * Loads all the node shapes.
 * 
 * @tag Shrimp.NodeShapes : this defines all the applicable {@link NodeShape} objects.
 * @author Chris Callendar
 * @date 5-Mar-07
 */
public class NodeShapes {

	/**
	 * Gets all the defined node shapes.
	 * @return array of the defined {@link NodeShape} objects
	 */
	public static NodeShape[] getNodeShapes() {
		Vector shapes = new Vector(12);
		shapes.add(new RectangleNodeShape());
		shapes.add(new StackedRectangleNodeShape());
		shapes.add(new DropShadowRectangleNodeShape());
		shapes.add(new RectangleTriangleNodeShape());
		shapes.add(new RoundedRectangleNodeShape());
		shapes.add(new EllipseNodeShape());
		shapes.add(new TriangleNodeShape());
		shapes.add(new InvertedTriangleNodeShape());
		shapes.add(new DiamondNodeShape());
		shapes.add(new ObjectLifeLineNodeShape());
		shapes.add(new FileNodeShape());
		shapes.add(new FolderNodeShape());
		shapes.add(new CylinderNodeShape());
		
		// @tag Shrimp.NodeShapes.Christmas : only show when December!
		if (Calendar.getInstance().get(Calendar.MONTH) == Calendar.DECEMBER) {
			Vector xmasShapes = ChristmasNodeShapes.getChristmasShapes();
			shapes.addAll(xmasShapes);
		}
		
		return (NodeShape[]) shapes.toArray(new NodeShape[shapes.size()]);
	}
	
	public static NodeShape[] getChristmasNodeShapes() {
		Vector xmasShapes = ChristmasNodeShapes.getChristmasShapes();
		return (NodeShape[]) xmasShapes.toArray(new NodeShape[xmasShapes.size()]);
	}
	
}
