/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ca.uvic.csr.shrimp.AttributeConstants;
import ca.uvic.csr.shrimp.VisVarConstants;
import ca.uvic.csr.shrimp.AttrToVisVarBean.AttrToVisVarBean;
import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DataDisplayBridge.DataDisplayBridge;
import ca.uvic.csr.shrimp.DataDisplayBridge.RelTypeGroup;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpCompositeArc;


class GroupOptionsPanel extends JPanel {
    private static final Color DEFAULT_COMPOSITE_COLOR = Color.RED;
	private RelTypeGroup relTypeGroup;
	private GroupPanel groupPanel;
	private AttrToVisVarBean attrToVisVarBean;
	private DataDisplayBridge dataDisplayBridge;
	private DataBean dataBean;
	
	private JCheckBox chkEnableComposites;
	private JLabel colorLabel;
	private JButton applyChangesButton;
	private JSlider highLevelSlider;
	
	public GroupOptionsPanel(GroupPanel groupPanel, AttrToVisVarBean attrToVisVarBean, DataDisplayBridge dataDisplayBridge, DataBean dataBean) {
		this.groupPanel = groupPanel;
		this.attrToVisVarBean = attrToVisVarBean;
		this.dataDisplayBridge = dataDisplayBridge;
		this.dataBean = dataBean;
		this.relTypeGroup = groupPanel.getRelTypeGroup();
		
		setOpaque(false);
	
		chkEnableComposites = new JCheckBox("Enable High Level Arcs?");
		chkEnableComposites.setOpaque(false);

		colorLabel = new JLabel();
		colorLabel.setBorder(BorderFactory.createRaisedBevelBorder());
		colorLabel.setPreferredSize(new Dimension(40, 20));
		colorLabel.setToolTipText("The color for composite arcs");
		colorLabel.setBackground(DEFAULT_COMPOSITE_COLOR);
		colorLabel.setOpaque(true);
		
		final JButton colorButton = new JButton("Choose Color");
		colorButton.setToolTipText("Click to change the composite arcs color...");
		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color newColor = JColorChooser.showDialog(colorLabel, "Choose Color", colorLabel.getBackground());
				if (newColor != null) {
					colorLabel.setBackground(newColor);
					applyChangesButton.setEnabled(true);
				}
			}
		});
		colorLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				colorButton.doClick();
			}
		});

		
		final JPanel colorPanel = new JPanel();
		colorPanel.add(new JLabel("Color:"));
		colorPanel.add(colorLabel);
		colorPanel.add(colorButton);
	
		applyChangesButton = new JButton("Apply Changes");
		applyChangesButton.setEnabled(false);
		applyChangesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ProgressDialog.showProgress();
				ProgressDialog.setSubtitle("Creating composites...");
				applyChanges();
				ProgressDialog.tryHideProgress();
			}
		});
	
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		this.setLayout(new GridBagLayout());
	
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.gridwidth = 3;
		gridBagConstraints.insets = new Insets(5, 5, 5, 5);
		gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
		this.add(chkEnableComposites, gridBagConstraints);
	
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.weightx = 1.0;
		this.add(colorPanel, gridBagConstraints);
		
		final JLabel highLevelSliderLabel = new JLabel("Filter by number of Arcs contained:");
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.gridwidth = 3;
		this.add(highLevelSliderLabel, gridBagConstraints);
	
		highLevelSlider = new JSlider(0, 100, 0);
		highLevelSlider.addChangeListener(new ChangeListener () {
			public void stateChanged(ChangeEvent e) {
				applyChangesButton.setEnabled(true);
			}
		});
		highLevelSlider.setOpaque(false);
		highLevelSlider.setMajorTickSpacing(10);
		highLevelSlider.setMinorTickSpacing(5);
		highLevelSlider.setPaintTicks(true);
		highLevelSlider.setPaintLabels(true);
		highLevelSlider.setSnapToTicks(true);
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 4;
		gridBagConstraints.gridwidth = 3;
		gridBagConstraints.weightx = 1.0;
		this.add(highLevelSlider, gridBagConstraints);
	
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 5;
		gridBagConstraints.gridwidth = 3;
		gridBagConstraints.anchor = GridBagConstraints.SOUTHWEST;
		gridBagConstraints.insets = new Insets(10, 5, 5, 5);
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		this.add(applyChangesButton, gridBagConstraints);
	
		colorLabel.setEnabled(false);
		colorPanel.setEnabled(false);
		highLevelSlider.setEnabled(false);
		highLevelSliderLabel.setEnabled(false);
	
		chkEnableComposites.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				JCheckBox box = (JCheckBox) e.getSource();
				colorLabel.setEnabled(box.isSelected());
				colorPanel.setEnabled(box.isSelected());
				highLevelSlider.setEnabled(box.isSelected());
				highLevelSliderLabel.setEnabled(box.isSelected());
				applyChangesButton.setEnabled(true);
			}
		});
	}

	private Color getColor() {
		return colorLabel.getBackground();
	}

	public void refresh() {
		if (relTypeGroup.areCompositesEnabled()) {
			chkEnableComposites.setSelected(true);
			colorLabel.setBackground(relTypeGroup.getCompositeColor());
		} else {
			chkEnableComposites.setSelected(false);
		}
		applyChangesButton.setEnabled(false);
	}

	protected void applyChanges() {
	    boolean firingEvents = dataBean.isFiringEvents();
	    dataBean.setFiringEvents(false);
	    
		if (chkEnableComposites.isSelected()) {
		    relTypeGroup.setCompositeColor(this.getColor());

			attrToVisVarBean.setNominalVisualVariableValue(AttributeConstants.NOM_ATTR_REL_TYPE, VisVarConstants.VIS_VAR_ARC_COLOR, relTypeGroup.getGroupName(), this.getColor());
			attrToVisVarBean.setNominalVisualVariableValue(AttributeConstants.NOM_ATTR_REL_TYPE, VisVarConstants.VIS_VAR_ARC_STYLE, relTypeGroup.getGroupName(), relTypeGroup.getCompositeStyle());

			// TODO get composite threshold working
			//int threshold = highLevelSlider.getValue(); 
			Collection arcs = dataDisplayBridge.getCompositeArcsManager().getCompositeArcs();
			for (Iterator iterator = arcs.iterator(); iterator.hasNext();) {
				ShrimpCompositeArc arc = (ShrimpCompositeArc) iterator.next();
				if (arc.isAtHighLevel()) {
					arc.updateVisibility();
					//if (arc.getArcCount() < threshold) {
						//displayBean.setVisible(arc, false, false);
					//} else {
						//displayBean.setVisible(arc, true, false);
					//}
				}
			}
		}

		groupPanel.setCompositeArcsEnabled(chkEnableComposites.isSelected());
		groupPanel.setCompositeArcsColor(getColor());
		applyChangesButton.setEnabled(false);
		
		dataBean.setFiringEvents(firingEvents);
	}
}