/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */

package ca.uvic.csr.shrimp.gui.options;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;

/**
 * A dialog box for changing label options.
 */
public class LabelOptionsPanel extends JPanel implements ShrimpOptions{
	private DisplayBean displayBean;
	private SelectorBean selectorBean;
	
	private String chosenMode;
	private boolean modeChanged = false;
	
	private Font chosenFont;	
	private boolean fontChanged = false;
	
	private int chosenLevels;
	private boolean levelsChanged = false;
	
	private int switchAtNum;
	private boolean chosenSwitchLabelling;
	private boolean switchLabellingChanged = false;
	
	private boolean chosenFade;
	private boolean fadeOutChanged = false;
	
	private boolean chosenOpaque;
	private boolean opaqueChanged = false;
	
	// GUI widgets
	private JPanel fontChooser;
	private JComboBox nameBox, sizeBox, styleBox;
	private JLabel sampleLabel;
	private JTextField levelsTF;
	private JTextField switchAtNumTF;
	private JCheckBox switchChk, fadeChk, opaqueChk;
	private JLabel showLbl;
	private JLabel levelsLbl;
	private JRadioButton scaledBySizeBtn;
	private JRadioButton fitToNodeBtn;
	 
	public LabelOptionsPanel (DisplayBean db, SelectorBean selectorBean) {
		displayBean = db;	
		this.selectorBean = selectorBean;
		fontChooser = createFontChooser();
		
		scaledBySizeBtn = new JRadioButton(DisplayConstants.SCALE_BY_NODE_SIZE_LONG_NAME);
		fitToNodeBtn = new JRadioButton(DisplayConstants.FIT_TO_NODE_SIZE_LONG_NAME);
		JRadioButton scaledByLevelBtn = new JRadioButton(DisplayConstants.SCALE_BY_LEVEL_LONG_NAME);
		JRadioButton fixedBtn = new JRadioButton(DisplayConstants.FIXED_LONG_NAME);	
		JRadioButton wrapToNodeBtn = new JRadioButton(DisplayConstants.WRAP_TO_NODE_SIZE_LONG_NAME);	
		ButtonGroup bGrp = new ButtonGroup ();
		bGrp.add(scaledBySizeBtn);
		bGrp.add(fitToNodeBtn);
		bGrp.add(wrapToNodeBtn);
		bGrp.add(scaledByLevelBtn);
		bGrp.add(fixedBtn);
		
		String currentMode = displayBean.getDefaultLabelMode();
		if (currentMode.equals(DisplayConstants.LABEL_MODE_SCALE_BY_NODE_SIZE)) {
			scaledBySizeBtn.setSelected(true);
		// @tag shrimp(fitToNodeLabelling)
		} else if (currentMode.equals(DisplayConstants.FIT_TO_NODE_SIZE_LONG_NAME)){
			fitToNodeBtn.setSelected(true);
		} else if (currentMode.equals(DisplayConstants.WRAP_TO_NODE_SIZE_LONG_NAME)){
			wrapToNodeBtn.setSelected(true);
		} else if (currentMode.equals(DisplayConstants.LABEL_MODE_SCALE_BY_LEVEL)){
			scaledByLevelBtn.setSelected(true);
		} else if (currentMode.equals(DisplayConstants.LABEL_MODE_FIXED)){
			fixedBtn.setSelected(true);
		} else {
			// nothing 
		}
		chosenMode = currentMode;
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(null);
		choicePanel.setPreferredSize(new Dimension(400,295));
		
		JLabel switchModeLbl = new JLabel("Label on node if nodes greater than: ");
		switchChk = new JCheckBox ("");
		chosenSwitchLabelling = displayBean.getSwitchLabelling();
		switchChk.setSelected(chosenSwitchLabelling);
		switchAtNum = displayBean.getSwitchAtNum();
		
		JLabel modeLbl = new JLabel("Select Mode: ");

		showLbl = new JLabel("Show at most ");
		levelsLbl = new JLabel(" level(s)");
		fadeChk = new JCheckBox ("Fade Out");
		chosenFade = displayBean.getLabelFadeOut();
		fadeChk.setSelected(chosenFade);
		
		opaqueChk = new JCheckBox ("Opaque Background");
		chosenOpaque = displayBean.getLabelBackgroundOpaque();
		opaqueChk.setSelected(chosenOpaque);
		
		chosenLevels = displayBean.getLabelLevels();
		levelsTF = new JTextField("" + chosenLevels, 2);
		GridBagLayout gridbag = new GridBagLayout();

		JPanel numlevelsPanel = new JPanel(gridbag);
		numlevelsPanel.add(showLbl);
		numlevelsPanel.add(levelsTF);
		numlevelsPanel.add(levelsLbl);
		
		JPanel levelOptionsPanel = new JPanel();
		levelOptionsPanel.setLayout(new BoxLayout(levelOptionsPanel, BoxLayout.Y_AXIS));
		
		levelOptionsPanel.add(numlevelsPanel);
		levelOptionsPanel.add(fadeChk);
		levelOptionsPanel.add(opaqueChk);
				
		choicePanel.add(scaledBySizeBtn);
		choicePanel.add(fitToNodeBtn);
		choicePanel.add(wrapToNodeBtn);
		choicePanel.add(scaledByLevelBtn);
		choicePanel.add(fixedBtn);
		choicePanel.add(modeLbl);
		
		JPanel switchLabelModePanel = new JPanel(new GridBagLayout());
		switchLabelModePanel.add(switchChk);
		switchLabelModePanel.add(switchModeLbl);
		switchAtNumTF = new JTextField("" + switchAtNum, 6);
		switchAtNumTF.setEnabled(chosenSwitchLabelling);
		switchLabelModePanel.add(switchAtNumTF);
		choicePanel.add(switchLabelModePanel);

		choicePanel.add(levelOptionsPanel);
		
		modeLbl.setSize(new Dimension(90,30));
		modeLbl.setHorizontalAlignment(SwingConstants.RIGHT);
		modeLbl.setLocation(10,10);
		
		scaledBySizeBtn.setSize(new Dimension(350,30));
		scaledBySizeBtn.setLocation(110,10);
		
		fitToNodeBtn.setSize(new Dimension(350,30));
		fitToNodeBtn.setLocation(110,35);
		
		wrapToNodeBtn.setSize(new Dimension(350,30));
		wrapToNodeBtn.setLocation(110,60);
		
		scaledByLevelBtn.setSize(new Dimension(350,30));
		scaledByLevelBtn.setLocation(110,85);
		
		fixedBtn.setSize(new Dimension(350,30));
		fixedBtn.setLocation(110,110);
		
		switchLabelModePanel.setBorder(BorderFactory.createEmptyBorder());
		switchLabelModePanel.setSize(new Dimension(300,70));
		switchLabelModePanel.setLocation(10, 125);
		
		levelOptionsPanel.setBorder(BorderFactory.createLineBorder(Color.black, 1));
		levelOptionsPanel.setSize(new Dimension(300,80));
		levelOptionsPanel.setLocation(25,195);
		
		scaledBySizeBtn.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenMode = DisplayConstants.LABEL_MODE_SCALE_BY_NODE_SIZE;
    			modeChanged = true;
    			updateLevelOptions();
    		}
    	});
		
		// @tag shrimp(fitToNodeLabelling)
		fitToNodeBtn.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenMode = DisplayConstants.LABEL_MODE_FIT_TO_NODE;
    			modeChanged = true;
    			updateLevelOptions();
    		}
    	});
		
		wrapToNodeBtn.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenMode = DisplayConstants.LABEL_MODE_WRAP_TO_NODE;
    			modeChanged = true;
    			updateLevelOptions();
    		}
    	});
		
		scaledByLevelBtn.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenMode = DisplayConstants.LABEL_MODE_SCALE_BY_LEVEL;
    			modeChanged = true;
    			updateLevelOptions();
    		}
    	});
		
		fixedBtn.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenMode = DisplayConstants.LABEL_MODE_FIXED;
    			modeChanged = true;
    			updateLevelOptions();
    		}
    	});
		
		fadeChk.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenFade = fadeChk.isSelected();
    			fadeOutChanged = true;
    		}
    	});
		opaqueChk.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenOpaque = opaqueChk.isSelected();
    			opaqueChanged = true;
    		}
    	});
		switchChk.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {
    			chosenSwitchLabelling = switchChk.isSelected();
    			switchAtNumTF.setEnabled(chosenSwitchLabelling);
    			switchLabellingChanged = true;
    		}
    	});
    	levelsTF.addKeyListener(new KeyAdapter () {
			public void keyReleased(KeyEvent e) {
				String text = levelsTF.getText();
				if (!text.equals("")) {
	   				chosenLevels = getChosenLevels();
	    			levelsChanged = true;
				}
			}
		});
		levelsTF.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {    			
    			chosenLevels = getChosenLevels();
    			levelsChanged = true;
    		}
    	});
    	switchAtNumTF.addKeyListener(new KeyAdapter () {
			public void keyReleased(KeyEvent e) {
				String text = switchAtNumTF.getText();
				if (!text.equals("")) {
					switchAtNum = getChosenSwitchAtNum();
					switchLabellingChanged = true;
				}
			}
		});
		levelsTF.addActionListener(new ActionListener() {			
    		public void actionPerformed(ActionEvent e) {    			
    			chosenLevels = getChosenSwitchAtNum();
    			switchLabellingChanged = true;
    		}
    	});
		
		this.add(choicePanel, BorderLayout.NORTH);
		this.add(fontChooser, BorderLayout.CENTER);
		
		updateLevelOptions();
	} 
	
	private void applyChange() {
		Cursor oldCursor = getCursor ();			
		setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
		if (fontChanged) {
			displayBean.setLabelFont (chosenFont);
			fontChanged = false;
		}
		if (modeChanged) {
			displayBean.setDefaultLabelMode (chosenMode);
			selectorBean.setSelected(DisplayConstants.LABEL_MODE, chosenMode);
			modeChanged = false;
		}
		if (fadeOutChanged) {
			displayBean.setLabelFadeOut(chosenFade);
			fadeOutChanged = false;
		}
		if (opaqueChanged) {
			displayBean.setLabelBackgroundOpaque(chosenOpaque);
			opaqueChanged = false;
		}
		if (switchLabellingChanged) {
			displayBean.setSwitchLabelling(chosenSwitchLabelling);
			displayBean.setSwitchAtNum(switchAtNum);
			switchLabellingChanged= false;
		}
		if (levelsChanged) {
			displayBean.setLabelLevels (chosenLevels);
			levelsChanged = false;
		}
		setCursor (oldCursor);			
	}
	
	private JPanel createFontChooser () {
		JPanel panel = new JPanel ();
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String fonts[] = ge.getAvailableFontFamilyNames();
        Vector fontNames = new Vector();
        for ( int i = 1; i < fonts.length; i++ ) {
            fontNames.addElement(fonts[i]);
        }
    
		nameBox = new JComboBox(fontNames);      
        sizeBox = new JComboBox( new Object[]{ 	"4", 
        										"6", 
        										"8", 
        										"10", 
        										"12", 
        										"14", 
        										"16", 
        										"18", 
        										"20", 
        										"22", 
        										"24", 
        										"26", 
        										"28", 
        										"30"} );
        sizeBox.setEditable(true);
        
        										
		styleBox = new JComboBox( new Object[]{"PLAIN", "BOLD", "ITALIC", "BOLD & ITALIC"} );		
		
		JPanel choicePanel = new JPanel (new GridLayout (1,3));
		choicePanel.add(nameBox);
		choicePanel.add(sizeBox);
		choicePanel.add(styleBox);
		
		JPanel samplePanel = new JPanel ();
		sampleLabel = new JLabel("Sample Text");
		sampleLabel.setHorizontalAlignment (SwingConstants.CENTER);
		sampleLabel.setVerticalAlignment (SwingConstants.CENTER);
		sampleLabel.setPreferredSize(new Dimension (200,80));
		samplePanel.add(sampleLabel);
		samplePanel.setBackground(Color.white);
			
		panel.setLayout(new BorderLayout());
		panel.setBorder(BorderFactory.createLineBorder(Color.black));
		panel.add(choicePanel, BorderLayout.NORTH);
		panel.add(samplePanel, BorderLayout.CENTER);

		// choose current font
		Font currentFont = (Font) displayBean.getLabelFont();
		nameBox.setSelectedItem(currentFont.getName());
		sizeBox.setSelectedItem(new Integer (currentFont.getSize()));
		styleBox.setSelectedIndex(currentFont.getStyle());
		sampleLabel.setFont(currentFont);
		chosenFont = currentFont;
		
		// add listener to each combo box
		ChoiceListener choiceListener = new ChoiceListener ();
		nameBox.addItemListener(choiceListener);
		sizeBox.addItemListener(choiceListener);
		styleBox.addItemListener(choiceListener);
		
        sizeBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	int size = getChosenFontSize();
		        fontChanged = true;
				changeFont((String)nameBox.getSelectedItem(), size, styleBox.getSelectedIndex());
            }
        });
		
		return panel;
	}

	private void updateLevelOptions () {
		boolean enable = !(scaledBySizeBtn.isSelected() ||
				fitToNodeBtn.isSelected());
		fadeChk.setEnabled(enable);
		opaqueChk.setEnabled(enable);
		showLbl.setEnabled(enable);
		levelsTF.setEnabled(enable);
		levelsLbl.setEnabled(enable);		
	}
	
	
	private void changeFont (String name, int size, int style) {
		Cursor oldCursor = getCursor ();			
		setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
		chosenFont = new Font (name, style, size);
		sampleLabel.setFont (chosenFont);
		setCursor (oldCursor);			
	}

	private class ChoiceListener implements ItemListener {
	    public void itemStateChanged(ItemEvent e) {
	        if ( e.getStateChange() != ItemEvent.SELECTED ) {
	            return;
	        }
	        fontChanged = true;
            int size = getChosenFontSize();
			changeFont((String)nameBox.getSelectedItem(), size, styleBox.getSelectedIndex());
	   	}
	} 
	
	private int getChosenFontSize () {
		int size = 0;
		try {
			Object sizeObj = sizeBox.getSelectedItem();
			if (sizeObj instanceof String) 			
				size = (Integer.valueOf((String)sizeBox.getSelectedItem())).intValue();
			else if (sizeObj instanceof Integer)
				size = ((Integer)sizeObj).intValue();
       	} catch (Exception ex) {
			String title = "Input Error";
			String msg = "\"" + (String)sizeBox.getSelectedItem() + "\" is not a valid size for labels";
			JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), msg, title, JOptionPane.ERROR_MESSAGE);
			sizeBox.setSelectedItem("" + chosenFont.getSize());
       	}
		return size;
	}
	
	private int getChosenLevels () {
		int levels = chosenLevels;
		try {
			levels = (Integer.valueOf(levelsTF.getText())).intValue();
       	} catch (Exception ex) {
			String title = "Input Error";
			String msg = "\"" + levelsTF.getText() + "\" is not a valid number of levels";
			JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), msg, title, JOptionPane.ERROR_MESSAGE);
			levelsTF.setText("" + chosenLevels);
       	}
		return levels;
	}
	
	private int getChosenSwitchAtNum () {
		int num = this.switchAtNum;
		try {
			num = (Integer.valueOf(switchAtNumTF.getText())).intValue();
       	} catch (Exception ex) {
			String title = "Input Error";
			String msg = "\"" + switchAtNumTF.getText() + "\" is not a valid number of levels";
			JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), msg, title, JOptionPane.ERROR_MESSAGE);
			switchAtNumTF.setText("" + this.switchAtNum);
       	}
		return num;
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#ok()
	 */
	public void ok() {
		applyChange();
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#cancel()
	 */
	public void cancel() {
	}
	
}